#include "mem_arena.h"
#include "mem_utils.h"
#include "misc/misc_utils.h"
#include "rasteriser/rasteriser.h"
#include "vector/vec.h"
#include "window/window.h"
#include <SDL2/SDL_events.h>
#include <SDL2/SDL_keycode.h>
#include <stdlib.h>

int main(void) {
  colour_t bg =
      (colour_t){.rgba.r = 27, .rgba.g = 38, .rgba.b = 79, .rgba.a = 255};
  window_t window = {0};

  if (!init_window(&window, 800, 800, "CG From Scratch Rasteriser")) {
    return EXIT_FAILURE;
  }

  bool running = true;
  SDL_Event event = {0};

  // i32 w_min = ((i32)window.half_width) * -1;
  // i32 w_max = (i32)window.half_width;
  // i32 h_min = ((i32)window.half_height) * -1;
  // i32 h_max = (i32)window.half_height;

  triangle_t triangle = {
      .p0 = {-200, -250},
      .p1 = {200, 50},
      .p2 = {20, 250},
      .h0 = 0,
      .h1 = 0,
      .h2 = 1,
  };

  Arena *arena = NULL;
  u64 capacity = 10ull * 1024ull * 1024ull * 1024ull;
  if (!wapp_mem_arena_init(&arena, capacity, WAPP_MEM_ALLOC_RESERVE, false)) {
    return EXIT_FAILURE;
  }

  while (running) {
    while (SDL_PollEvent(&event)) {
      switch (event.type) {
      case SDL_QUIT:
        running = false;
        break;
      }
    }

    clear_window(&window, bg);

    draw_shaded_triangle(&window, arena, triangle,
                         (colour_t){.colour = 0x00ff00ff});
    draw_wireframe_triangle(&window, arena, triangle,
                            (colour_t){.colour = 0xff008aff});

    swap_buffers(&window);

    wapp_mem_arena_clear(arena);
  }

  wapp_mem_arena_destroy(&arena);
  close_window(&window);

  return EXIT_SUCCESS;
}
